@@ -574,6 +574,7 @@ _check_file (GFile *mount_root, const char *file_path,
 	return _check_file_common (file, must_be_executable);
 }
 
+#ifdef NAUTILUS_AUTORUN_SUPPORTS_AUTORUN_EXE
 static gboolean
 _check_file_case_insensitive (GFile *mount_root, const char *file_path,
                               gboolean must_be_executable)
@@ -582,6 +583,7 @@ _check_file_case_insensitive (GFile *mount_root, const char *file_path,
 	GFile *file = nautilus_find_file_insensitive (mount_root, file_path);
 	return _check_file_common (file, must_be_executable);
 }
+#endif
 
 /**
  * _g_mount_guess_content_type:
@@ -729,18 +731,24 @@ _g_mount_guess_content_type (GMount              *mount,
 		g_ptr_array_add (types, g_strdup ("x-content/image-picturecd"));
 	}
 
-	if (_check_file (root, ".autorun", TRUE) ||
-	    _check_file (root, "autorun", TRUE) ||
-	    _check_file (root, "autorun.sh", TRUE) ||
-	    _check_file_case_insensitive (root, "autorun.exe", TRUE) ||
-	    _check_file_case_insensitive (root, "autorun.inf", FALSE)) {
+	if (g_file_is_native (root) &&
+	    (_check_file (root, ".autorun", TRUE) ||
+	     _check_file (root, "autorun", TRUE) ||
+	     _check_file (root, "autorun.sh", TRUE) ||
+#ifdef NAUTILUS_AUTORUN_SUPPORTS_AUTORUN_EXE
+	     /* TODO */
+	     _check_file_case_insensitive (root, "autorun.exe", TRUE) ||
+	     _check_file_case_insensitive (root, "autorun.inf", FALSE)))
+#else
+	    0))
+#endif
+	{
 		/* http://standards.freedesktop.org/autostart-spec/autostart-spec-latest.html */
-
+		
 		/* http://bugzilla.gnome.org/show_bug.cgi?id=509823#c3 for the autorun.exe and autorun.inf stuff */
 		g_ptr_array_add (types, g_strdup ("x-content/software"));
 	}
 
-
 no_sniff:
 	
 	g_ptr_array_add (types, NULL);
@@ -848,6 +856,10 @@ _g_mount_guess_content_type_finish (GMount              *mount,
 
 /*- END MOVE TO GIO ---*/
 
+enum {
+	AUTORUN_DIALOG_RESPONSE_EJECT = 0
+};
+
 typedef struct
 {
 	GtkWidget *dialog;
@@ -915,6 +927,13 @@ static void
 autorun_dialog_response (GtkDialog *dialog, gint response, AutorunDialogData *data)
 {
 	switch (response) {
+	case AUTORUN_DIALOG_RESPONSE_EJECT:
+		nautilus_file_operations_unmount_mount (GTK_WINDOW (dialog),
+							data->mount,
+							data->should_eject,
+							FALSE);
+		break;
+
 	case GTK_RESPONSE_NONE:
 		/* window was closed */
 		break;
@@ -1154,7 +1173,7 @@ show_dialog:
 		eject_button = gtk_button_new_with_mnemonic (_("_Unmount"));
 		data->should_eject = FALSE;
 	}
-	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), eject_button, 0);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), eject_button, AUTORUN_DIALOG_RESPONSE_EJECT);
 	gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (GTK_DIALOG (dialog)->action_area), eject_button, TRUE);
 
 	/* show the dialog */
@@ -1258,112 +1277,156 @@ nautilus_autorun (GMount *mount, NautilusAutorunOpenWindow open_window_func, gpo
 					   data);
 }
 
-char **
-nautilus_autorun_get_x_content_types_for_mount (GMount      *mount,
-						gboolean     force_rescan)
+typedef struct {
+	NautilusAutorunGetContent callback;
+	gpointer user_data;
+} GetContentTypesData;
+
+static void
+get_types_cb (GObject *source_object,
+	      GAsyncResult *res,
+	      gpointer user_data)
 {
+	GetContentTypesData *data;
+	char **types;
+
+	data = user_data;
+	types = _g_mount_guess_content_type_finish (G_MOUNT (source_object), res, NULL);
+
+	if (data->callback) {
+		data->callback (types, data->user_data);
+	}
+	g_strfreev (types);
+	g_free (data);
+}
+
+void
+nautilus_autorun_get_x_content_types_for_mount_async (GMount *mount,
+						      NautilusAutorunGetContent callback,
+						      GCancellable *cancellable,
+						      gpointer user_data)
+{
+	char **cached;
+	GetContentTypesData *data;
+	
+	if (mount == NULL) {
+		if (callback) {
+			callback (NULL, user_data);
+		}
+		return;
+	}
+
+	cached = g_object_get_data (G_OBJECT (mount), "content-type-cache");
+	if (cached != NULL) {
+		if (callback) {
+			callback (cached, user_data);
+		}
+		return;
+	}
+
+	data = g_new (GetContentTypesData, 1);
+	data->callback = callback;
+	data->user_data = user_data;
+	
+	_g_mount_guess_content_type_async (mount,
+					   FALSE,
+					   cancellable,
+					   get_types_cb,
+					   data);
+}
+
+
+char **
+nautilus_autorun_get_cached_x_content_types_for_mount (GMount      *mount)
+{
+	char **cached;
+	
 	if (mount == NULL) {
 		return NULL;
 	}
 
-	/* since we always guess the content type at mount type, we're guaranteed
-	 * to get the cached results
-	 *
-	 * TODO: Really? what if we didn't mount the mount ourself?
-	 */
-	return _g_mount_guess_content_type (mount, force_rescan, NULL);
-}
-
-
-static GList *inhibit_mount_handling_for = NULL;
-
-
-static gboolean
-remove_inhibit_file_cb (gpointer data)
-{
-	GFile *file = data;
-	GList *l;
-
-	l = g_list_find (inhibit_mount_handling_for, file);
-	if (l != NULL) {
-		inhibit_mount_handling_for = g_list_delete_link (inhibit_mount_handling_for, l);
-		g_object_unref (file);
+	cached = g_object_get_data (G_OBJECT (mount), "content-type-cache");
+	if (cached != NULL) {
+		return g_strdupv (cached);
 	}
-	
-	return FALSE;
-}
 
-void
-nautilus_inhibit_autorun_for_file (GFile *file)
-{
-	inhibit_mount_handling_for = g_list_prepend (inhibit_mount_handling_for, g_object_ref (file));
-	g_timeout_add_full (0,
-			    5000,
-			    remove_inhibit_file_cb,
-			    g_object_ref (file),
-			    g_object_unref);
+	return NULL;
 }
 
 static gboolean
-remove_inhibit_volume (gpointer data)
+remove_allow_volume (gpointer data)
 {
 	GVolume *volume = data;
 
-	g_object_set_data (G_OBJECT (volume), "nautilus-inhibit-autorun", NULL);
+	g_object_set_data (G_OBJECT (volume), "nautilus-allow-autorun", NULL);
 	return FALSE;
 }
 
 void
-nautilus_inhibit_autorun_for_volume (GVolume *volume)
+nautilus_allow_autorun_for_volume (GVolume *volume)
 {
-	g_object_set_data (G_OBJECT (volume), "nautilus-inhibit-autorun", GINT_TO_POINTER (1));
-	g_timeout_add_full (0,
-			    5000,
-			    remove_inhibit_volume,
-			    g_object_ref (volume),
-			    g_object_unref);
+	g_object_set_data (G_OBJECT (volume), "nautilus-allow-autorun", GINT_TO_POINTER (1));
+}
+
+void
+nautilus_allow_autorun_for_volume_finish (GVolume *volume)
+{
+	if (g_object_get_data (G_OBJECT (volume), "nautilus-allow-autorun") != NULL) {
+		g_timeout_add_full (0,
+				    5000,
+				    remove_allow_volume,
+				    g_object_ref (volume),
+				    g_object_unref);
+	}
+}
+
+static gboolean
+should_skip_native_mount_root (GFile *root)
+{
+	char *path;
+	gboolean should_skip;
+
+	/* skip any mounts in hidden directory hierarchies */
+	path = g_file_get_path (root);
+	should_skip = strstr (path, "/.") != NULL;
+	g_free (path);
+
+	return should_skip;
 }
 
 static gboolean
 should_autorun_mount (GMount *mount)
 {
-	GFile *root, *file;
-	GList *l;
+	GFile *root;
 	GVolume *enclosing_volume;
 	gboolean ignore_autorun;
 
-	ignore_autorun = FALSE;
+	ignore_autorun = TRUE;
 	enclosing_volume = g_mount_get_volume (mount);
 	if (enclosing_volume != NULL) {
-		if (g_object_get_data (G_OBJECT (enclosing_volume), "nautilus-inhibit-autorun") != NULL) {
-			ignore_autorun = TRUE;
-			g_object_set_data (G_OBJECT (enclosing_volume), "nautilus-inhibit-autorun", NULL);
+		if (g_object_get_data (G_OBJECT (enclosing_volume), "nautilus-allow-autorun") != NULL) {
+			ignore_autorun = FALSE;
+			g_object_set_data (G_OBJECT (enclosing_volume), "nautilus-allow-autorun", NULL);
 		}
-		g_object_unref (enclosing_volume);
 	}
 
 	if (ignore_autorun) {
+		if (enclosing_volume != NULL) {
+			g_object_unref (enclosing_volume);
+		}
 		return FALSE;
 	}
 	
 	root = g_mount_get_root (mount);
 
-	for (l = inhibit_mount_handling_for; l != NULL; l = l->next) {
-		file = l->data;
-		if (g_file_contains_file (root, file)) {
-			ignore_autorun = TRUE;
-			
-			inhibit_mount_handling_for = g_list_delete_link (inhibit_mount_handling_for, l);
-			g_object_unref (file);
-			
-			break;
-		}
+	/* only do autorun on local files or files where g_volume_should_automount() returns TRUE */
+	ignore_autorun = TRUE;
+	if ((g_file_is_native (root) && !should_skip_native_mount_root (root)) || 
+	    (enclosing_volume != NULL && g_volume_should_automount (enclosing_volume))) {
+		ignore_autorun = FALSE;
 	}
-
-	if (!g_file_is_native (root)) {
-		/* only do autorun on local files */
-		/* TODO: Maybe we should do this on some gvfs mounts? like gphoto: ? */
-		ignore_autorun = TRUE;
+	if (enclosing_volume != NULL) {
+		g_object_unref (enclosing_volume);
 	}
 	g_object_unref (root);
 
