@@ -30,11 +30,11 @@
 #include "fm-tree-model.h"
 
 #include <eel/eel-glib-extensions.h>
+#include <eel/eel-gdk-pixbuf-extensions.h>
 #include <glib/gi18n.h>
 #include <libnautilus-private/nautilus-directory.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-file.h>
-#include <libnautilus-private/nautilus-icon-factory.h>
 #include <gtk/gtkenums.h>
 #include <string.h>
 
@@ -61,8 +61,8 @@ struct TreeNode {
 
 	NautilusFile *file;
 	char *display_name;
-	char *icon_name;
-	GnomeVFSVolume *volume;
+	GIcon *icon;
+	GMount *mount;
 	GdkPixbuf *closed_pixbuf;
 	GdkPixbuf *open_pixbuf;
 	GdkPixbuf *emblem_pixbuf;
@@ -124,7 +124,7 @@ static void destroy_node_without_reporting (FMTreeModel *model,
 static void report_node_contents_changed   (FMTreeModel *model,
 					    TreeNode          *node);
 
-static guint
+static GtkTreeModelFlags 
 fm_tree_model_get_flags (GtkTreeModel *tree_model)
 {
 	return GTK_TREE_MODEL_ITERS_PERSIST;
@@ -203,7 +203,7 @@ tree_node_destroy (FMTreeModel *model, TreeNode *node)
 
 	g_object_unref (node->file);
 	g_free (node->display_name);
-	g_free (node->icon_name);
+	object_unref_if_not_NULL (node->icon);
 	object_unref_if_not_NULL (node->closed_pixbuf);
 	object_unref_if_not_NULL (node->open_pixbuf);
 	object_unref_if_not_NULL (node->emblem_pixbuf);
@@ -241,29 +241,59 @@ tree_node_parent (TreeNode *node, TreeNode *parent)
 }
 
 static GdkPixbuf *
-tree_node_get_pixbuf_from_factory (TreeNode *node,
-				   const char *modifier)
+get_menu_icon (GIcon *icon)
+{
+	NautilusIconInfo *info;
+	GdkPixbuf *pixbuf;
+	int size;
+
+	size = nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU);
+	
+	info = nautilus_icon_info_lookup (icon, size);
+	pixbuf = nautilus_icon_info_get_pixbuf_nodefault_at_size (info, size);
+	g_object_unref (info);
+	
+	return pixbuf;
+}
+
+static GdkPixbuf *
+get_menu_icon_for_file (NautilusFile *file,
+			NautilusFileIconFlags flags)
+{
+	NautilusIconInfo *info;
+	GdkPixbuf *pixbuf;
+	int size;
+
+	size = nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU);
+	
+	info = nautilus_file_get_icon (file, size, flags);
+	pixbuf = nautilus_icon_info_get_pixbuf_nodefault_at_size (info, size);
+	g_object_unref (info);
+	
+	return pixbuf;
+}
+
+static GdkPixbuf *
+tree_node_get_pixbuf (TreeNode *node,
+		      NautilusFileIconFlags flags)
 {
 	if (node->parent == NULL) {
-		return nautilus_icon_factory_get_pixbuf_from_name_with_stock_size
-			(node->icon_name, NULL,
-			 GTK_ICON_SIZE_MENU, NULL);
+		return get_menu_icon (node->icon);
 	}
-	return nautilus_icon_factory_get_pixbuf_for_file_with_stock_size
-		(node->file, modifier, GTK_ICON_SIZE_MENU);
+	return get_menu_icon_for_file (node->file, flags);
 }
 
 static gboolean
 tree_node_update_pixbuf (TreeNode *node,
 			 GdkPixbuf **pixbuf_storage,
-			 const char *modifier)
+			 NautilusFileIconFlags flags)
 {
 	GdkPixbuf *pixbuf;
 
 	if (*pixbuf_storage == NULL) {
 		return FALSE;
 	}
-	pixbuf = tree_node_get_pixbuf_from_factory (node, modifier);
+	pixbuf = tree_node_get_pixbuf (node, flags);
 	if (pixbuf == *pixbuf_storage) {
 		g_object_unref (pixbuf);
 		return FALSE;
@@ -276,42 +306,47 @@ tree_node_update_pixbuf (TreeNode *node,
 static gboolean
 tree_node_update_closed_pixbuf (TreeNode *node)
 {
-	return tree_node_update_pixbuf (node, &node->closed_pixbuf, NULL);
+	return tree_node_update_pixbuf (node, &node->closed_pixbuf, 0);
 }
 
 static gboolean
 tree_node_update_open_pixbuf (TreeNode *node)
 {
-	return tree_node_update_pixbuf (node, &node->open_pixbuf, "accept");
+	return tree_node_update_pixbuf (node, &node->open_pixbuf, NAUTILUS_FILE_ICON_FLAGS_FOR_OPEN_FOLDER);
 }
 
 static GdkPixbuf *
-tree_node_get_emblem_pixbuf_from_factory (TreeNode *node)
+tree_node_get_emblem_pixbuf_internal (TreeNode *node)
 {
 	GdkPixbuf *pixbuf;
-	GList *emblem_icons;
-	EelStringList *emblems_to_ignore;
-
-	emblems_to_ignore = eel_string_list_new_from_string (NAUTILUS_FILE_EMBLEM_NAME_TRASH, TRUE);
+	GList *emblem_pixbufs;
+	char *emblems_to_ignore[3];
+	int i;
+	
+	i = 0;
+	emblems_to_ignore[i++] = NAUTILUS_FILE_EMBLEM_NAME_TRASH;
+	
 	if (node->parent && node->parent->file) {
 		if (!nautilus_file_can_write (node->parent->file)) {
-			eel_string_list_prepend (emblems_to_ignore, NAUTILUS_FILE_EMBLEM_NAME_CANT_WRITE);
+			emblems_to_ignore[i++] = NAUTILUS_FILE_EMBLEM_NAME_CANT_WRITE;
 		}
 	}
-	emblem_icons = nautilus_icon_factory_get_emblem_icons_for_file
-		(node->file, emblems_to_ignore);
-	eel_string_list_free (emblems_to_ignore);
+	
+	emblems_to_ignore[i++] = NULL;
 
-	if (emblem_icons != NULL) {
-		pixbuf = nautilus_icon_factory_get_pixbuf_for_icon_with_stock_size
-			(emblem_icons->data, NULL,
-			 GTK_ICON_SIZE_MENU,
-			 NULL, NULL, FALSE, NULL);
+	emblem_pixbufs = nautilus_file_get_emblem_pixbufs (node->file,
+							   nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU),
+							   TRUE,
+							   emblems_to_ignore);
+	
+
+	if (emblem_pixbufs != NULL) {
+		pixbuf = g_object_ref (emblem_pixbufs->data);
 	} else {
 		pixbuf = NULL;
 	}
 
-	eel_g_list_free_deep (emblem_icons);
+	eel_gdk_pixbuf_list_free (emblem_pixbufs);
 
 	return pixbuf;
 }
@@ -324,7 +359,7 @@ tree_node_update_emblem_pixbuf (TreeNode *node)
 	if (node->emblem_pixbuf == NULL) {
 		return FALSE;
 	}
-	pixbuf = tree_node_get_emblem_pixbuf_from_factory (node);
+	pixbuf = tree_node_get_emblem_pixbuf_internal (node);
 	if (pixbuf == node->emblem_pixbuf) {
 		g_object_unref (pixbuf);
 		return FALSE;
@@ -360,7 +395,7 @@ static GdkPixbuf *
 tree_node_get_closed_pixbuf (TreeNode *node)
 {
 	if (node->closed_pixbuf == NULL) {
-		node->closed_pixbuf = tree_node_get_pixbuf_from_factory (node, NULL);
+		node->closed_pixbuf = tree_node_get_pixbuf (node, 0);
 	}
 	return node->closed_pixbuf;
 }
@@ -369,7 +404,7 @@ static GdkPixbuf *
 tree_node_get_open_pixbuf (TreeNode *node)
 {
 	if (node->open_pixbuf == NULL) {
-		node->open_pixbuf = tree_node_get_pixbuf_from_factory (node, "accept");
+		node->open_pixbuf = tree_node_get_pixbuf (node, NAUTILUS_FILE_ICON_FLAGS_FOR_OPEN_FOLDER);
 	}
 	return node->open_pixbuf;
 }
@@ -378,7 +413,7 @@ static GdkPixbuf *
 tree_node_get_emblem_pixbuf (TreeNode *node)
 {
 	if (node->emblem_pixbuf == NULL) {
-		node->emblem_pixbuf = tree_node_get_emblem_pixbuf_from_factory (node);
+		node->emblem_pixbuf = tree_node_get_emblem_pixbuf_internal (node);
 	}
 	return node->emblem_pixbuf;
 }
@@ -489,7 +524,7 @@ create_node_for_file (FMTreeModelRoot *root, NautilusFile *file)
 	return node;
 }
 
-#if LOG_REF_COUNTS
+#ifdef LOG_REF_COUNTS
 
 static char *
 get_node_uri (GtkTreeIter *iter)
@@ -525,7 +560,7 @@ abandon_node_ref_count (FMTreeModel *model, TreeNode *node)
 {
 	if (node->parent != NULL) {
 		decrement_ref_count (model, node->parent, node->ref_count);
-#if LOG_REF_COUNTS
+#ifdef LOG_REF_COUNTS
 		if (node->ref_count != 0) {
 			char *uri;
 
@@ -544,7 +579,7 @@ abandon_dummy_row_ref_count (FMTreeModel *model, TreeNode *node)
 {
 	decrement_ref_count (model, node, node->dummy_child_ref_count);
 	if (node->dummy_child_ref_count != 0) {
-#if LOG_REF_COUNTS
+#ifdef LOG_REF_COUNTS
 		char *uri;
 
 		uri = nautilus_file_get_uri (node->file);
@@ -632,12 +667,13 @@ report_node_inserted (FMTreeModel *model, TreeNode *node)
 	report_row_inserted (model, &iter);
 	node->inserted = TRUE;
 
+	if (tree_node_has_dummy_child (node)) {
+		report_dummy_row_inserted (model, node);
+	}
+
 	if (node->directory != NULL ||
 	    node->parent == NULL) {
 		report_row_has_child_toggled (model, &iter);
-       }
-       if (tree_node_has_dummy_child (node)) {
-               report_dummy_row_inserted (model, node);
 	}
 }
 
@@ -1035,9 +1071,10 @@ get_tree_monitor_attributes (void)
 {
 	NautilusFileAttributes attributes;
 
-	attributes = nautilus_icon_factory_get_required_file_attributes ();
-	attributes |= NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY |
-		NAUTILUS_FILE_ATTRIBUTE_DISPLAY_NAME;
+	attributes =
+		NAUTILUS_FILE_ATTRIBUTES_FOR_ICON |
+		NAUTILUS_FILE_ATTRIBUTE_INFO |
+		NAUTILUS_FILE_ATTRIBUTE_LINK_INFO;
 	
 	return attributes;
 }
@@ -1475,7 +1512,7 @@ static void
 fm_tree_model_ref_node (GtkTreeModel *model, GtkTreeIter *iter)
 {
 	TreeNode *node, *parent;
-#if LOG_REF_COUNTS
+#ifdef LOG_REF_COUNTS
 	char *uri;
 #endif
 
@@ -1501,7 +1538,7 @@ fm_tree_model_ref_node (GtkTreeModel *model, GtkTreeIter *iter)
 			}
 			schedule_monitoring_update (FM_TREE_MODEL (model));
 		}
-#if LOG_REF_COUNTS
+#ifdef LOG_REF_COUNTS
 		uri = get_node_uri (iter);
 		g_message ("ref of %s, count is now %d",
 			   uri, parent->all_children_ref_count);
@@ -1514,7 +1551,7 @@ static void
 fm_tree_model_unref_node (GtkTreeModel *model, GtkTreeIter *iter)
 {
 	TreeNode *node, *parent;
-#if LOG_REF_COUNTS
+#ifdef LOG_REF_COUNTS
 	char *uri;
 #endif
 
@@ -1534,7 +1571,7 @@ fm_tree_model_unref_node (GtkTreeModel *model, GtkTreeIter *iter)
 
 	if (parent != NULL) {
 		g_assert (parent->all_children_ref_count > 0);
-#if LOG_REF_COUNTS
+#ifdef LOG_REF_COUNTS
 		uri = get_node_uri (iter);
 		g_message ("unref of %s, count is now %d",
 			   uri, parent->all_children_ref_count - 1);
@@ -1547,21 +1584,20 @@ fm_tree_model_unref_node (GtkTreeModel *model, GtkTreeIter *iter)
 }
 
 void
-fm_tree_model_add_root_uri (FMTreeModel *model, const char *root_uri, const char *display_name, const char *icon_name, GnomeVFSVolume *volume)
+fm_tree_model_add_root_uri (FMTreeModel *model, const char *root_uri, const char *display_name, GIcon *icon, GMount *mount)
 {
 	NautilusFile *file;
 	TreeNode *node, *cnode;
 	FMTreeModelRoot *newroot;
 	
-	file = nautilus_file_get (root_uri);
+	file = nautilus_file_get_by_uri (root_uri);
 
 	newroot = tree_model_root_new (model);
 	node = create_node_for_file (newroot, file);
 	node->display_name = g_strdup (display_name);
-	node->icon_name = g_strdup (icon_name);
-	if (volume) {
-		gnome_vfs_volume_ref (volume);
-		node->volume = volume;
+	node->icon = g_object_ref (icon);
+	if (mount) {
+		node->mount = g_object_ref (mount);
 	}
 	newroot->root_node = node;
 	node->parent = NULL;
@@ -1580,8 +1616,8 @@ fm_tree_model_add_root_uri (FMTreeModel *model, const char *root_uri, const char
 	report_node_inserted (model, node);
 }
 
-GnomeVFSVolume *
-fm_tree_model_get_volume_for_root_node_file (FMTreeModel *model, NautilusFile *file)
+GMount *
+fm_tree_model_get_mount_for_root_node_file (FMTreeModel *model, NautilusFile *file)
 {
 	TreeNode *node;
 
@@ -1592,7 +1628,7 @@ fm_tree_model_get_volume_for_root_node_file (FMTreeModel *model, NautilusFile *f
 	}
 
 	if (node) {
-		return node->volume;
+		return node->mount;
 	}
 
 	return NULL;
@@ -1606,7 +1642,7 @@ fm_tree_model_remove_root_uri (FMTreeModel *model, const char *uri)
 	FMTreeModelRoot *root;
 	NautilusFile *file;
 
-	file = nautilus_file_get (uri);
+	file = nautilus_file_get_by_uri (uri);
 	for (node = model->details->root_node; node != NULL; node = node->next) {
 		if (file == node->file) {
 			break;
@@ -1617,9 +1653,9 @@ fm_tree_model_remove_root_uri (FMTreeModel *model, const char *uri)
 	if (node) {
 		/* remove the node */
 		
-		if (node->volume) {
-			gnome_vfs_volume_unref (node->volume);
-			node->volume = NULL;
+		if (node->mount) {
+			g_object_unref (node->mount);
+			node->mount = NULL;
 		}
 
 		nautilus_file_monitor_remove (node->file, model);
@@ -1657,36 +1693,6 @@ fm_tree_model_new (void)
 	return model;
 }
 
-static void
-set_theme (TreeNode *node, FMTreeModel *model)
-{
-	TreeNode *child;
-	
-	tree_node_update_closed_pixbuf (node);
-	tree_node_update_open_pixbuf (node);
-	
-	report_node_contents_changed (model, node);
-	
-	for (child = node->first_child; child != NULL; child = child->next) {
-		set_theme (child, model);
-	}
-} 
-
-void
-fm_tree_model_set_theme (FMTreeModel *model)
-{
-	TreeNode *node;
-
-	g_return_if_fail (FM_IS_TREE_MODEL (model));
-
-	node = model->details->root_node;
-	while (node != NULL) {
-		set_theme (node, model);
-		node = node->next;
-	}
-}
-
-
 void
 fm_tree_model_set_show_hidden_files (FMTreeModel *model,
 					   gboolean show_hidden_files)
@@ -1762,6 +1768,40 @@ fm_tree_model_iter_get_file (FMTreeModel *model, GtkTreeIter *iter)
 	return node == NULL ? NULL : nautilus_file_ref (node->file);
 }
 
+/* This is used to work around some sort order stability problems
+   with gtktreemodelsort */
+int
+fm_tree_model_iter_compare_roots (FMTreeModel *model,
+				  GtkTreeIter *iter_a,
+				  GtkTreeIter *iter_b)
+{
+	TreeNode *a, *b, *n;
+
+	g_return_val_if_fail (FM_IS_TREE_MODEL (model), 0);
+	g_return_val_if_fail (iter_is_valid (model, iter_a), 0);
+	g_return_val_if_fail (iter_is_valid (model, iter_b), 0);
+	
+	a = iter_a->user_data;
+	b = iter_b->user_data;
+	
+	g_assert (a != NULL && a->parent == NULL);
+	g_assert (b != NULL && b->parent == NULL);
+
+	if (a == b) {
+		return 0;
+	}
+	
+	for (n = model->details->root_node; n != NULL; n = n->next) {
+		if (n == a) {
+			return -1;
+		}
+		if (n == b) {
+			return 1;
+		}
+	}
+	g_assert_not_reached ();
+}
+
 gboolean
 fm_tree_model_iter_is_root (FMTreeModel *model, GtkTreeIter *iter)
 {
@@ -1878,7 +1918,7 @@ fm_tree_model_get_type (void)
 	static GType object_type = 0;
 
 	if (object_type == 0) {
-		static const GTypeInfo object_info = {
+		const GTypeInfo object_info = {
 			sizeof (FMTreeModelClass),
 			NULL,
 			NULL,
@@ -1890,7 +1930,7 @@ fm_tree_model_get_type (void)
 			(GInstanceInitFunc) fm_tree_model_init,
 		};
 
-		static const GInterfaceInfo tree_model_info = {
+		const GInterfaceInfo tree_model_info = {
 			(GInterfaceInitFunc) fm_tree_model_tree_model_init,
 			NULL,
 			NULL
