@@ -31,7 +31,7 @@
 #include "nautilus-zoom-control.h"
 
 #include <atk/atkaction.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <eel/eel-accessibility.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-graphic-effects.h>
@@ -62,8 +62,8 @@ enum {
 struct NautilusZoomControlDetails {
 	GtkWidget *zoom_in;
 	GtkWidget *zoom_out;
-	GtkWidget *zoom_event;
 	GtkWidget *zoom_label;
+	GtkWidget *zoom_button;
 	
 	NautilusZoomLevel zoom_level;
 	NautilusZoomLevel min_zoom_level;	 
@@ -114,11 +114,94 @@ static void
 nautilus_zoom_control_finalize (GObject *object)
 {
 	g_list_free (NAUTILUS_ZOOM_CONTROL (object)->details->preferred_zoom_levels);
-	g_free (NAUTILUS_ZOOM_CONTROL (object)->details);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void 
+zoom_button_clicked (GtkButton *button, NautilusZoomControl *zoom_control)
+{
+	g_signal_emit (zoom_control, signals[ZOOM_TO_DEFAULT], 0);
+}
+
+static void
+zoom_popup_menu_show (GdkEventButton *event, NautilusZoomControl *zoom_control)
+{
+	eel_pop_up_context_menu (create_zoom_menu (zoom_control),
+				 EEL_DEFAULT_POPUP_MENU_DISPLACEMENT,
+				 EEL_DEFAULT_POPUP_MENU_DISPLACEMENT,
+				 event);
+}
+
+static void
+menu_position_under_widget (GtkMenu   *menu,
+			    gint      *x,
+			    gint      *y,
+			    gboolean  *push_in,
+			    gpointer   user_data)
+{
+	GtkWidget *widget;
+	GtkWidget *container;
+	GtkRequisition req;
+	GtkRequisition menu_req;
+	GdkRectangle monitor;
+	int monitor_num;
+	GdkScreen *screen;
+
+	widget = GTK_WIDGET (user_data);
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+
+	container = gtk_widget_get_ancestor (widget, GTK_TYPE_CONTAINER);
+	g_return_if_fail (container != NULL);
+
+	gtk_widget_size_request (widget, &req);
+	gtk_widget_size_request (GTK_WIDGET (menu), &menu_req);
+
+	screen = gtk_widget_get_screen (GTK_WIDGET (menu));
+	monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+	if (monitor_num < 0) {
+		monitor_num = 0;
+	}
+	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+
+	gdk_window_get_origin (widget->window, x, y);
+	if (GTK_WIDGET_NO_WINDOW (widget)) {
+		*x += widget->allocation.x;
+		*y += widget->allocation.y;
+	}
+
+	if (gtk_widget_get_direction (container) == GTK_TEXT_DIR_LTR) {
+		*x += widget->allocation.width - req.width;
+	} else {
+		*x += req.width - menu_req.width;
+	}
+
+	if ((*y + widget->allocation.height + menu_req.height) <= monitor.y + monitor.height) {
+		*y += widget->allocation.height;
+	} else if ((*y - menu_req.height) >= monitor.y) {
+		*y -= menu_req.height;
+	} else if (monitor.y + monitor.height - (*y + widget->allocation.height) > *y) {
+		*y += widget->allocation.height;
+	} else {
+		*y -= menu_req.height;
+	}
+
+	*push_in = FALSE;
+}
+
+
+static void
+zoom_popup_menu (GtkWidget *widget, NautilusZoomControl *zoom_control)
+{
+	GtkMenu *menu;
+
+	menu = create_zoom_menu (zoom_control);
+	gtk_menu_popup (menu, NULL, NULL,
+			menu_position_under_widget, widget,
+			0, gtk_get_current_event_time ());
+	gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
+}
+
 /* handle button presses */
 static gboolean
 nautilus_zoom_control_button_press_event (GtkWidget *widget,
@@ -129,19 +212,11 @@ nautilus_zoom_control_button_press_event (GtkWidget *widget,
 		return FALSE;
 	}
 
-	gtk_widget_grab_focus (widget);
-
-	/* check for the context menu button and handle by creating and showing the menu */
+	/* check for the context menu button and show the menu */
 	if (event->button == CONTEXTUAL_MENU_BUTTON) {
-		eel_pop_up_context_menu (create_zoom_menu (zoom_control),
-					 EEL_DEFAULT_POPUP_MENU_DISPLACEMENT,
-					 EEL_DEFAULT_POPUP_MENU_DISPLACEMENT,
-					 event);
+		zoom_popup_menu_show (event, zoom_control);
 		return TRUE;
-	} else if (event->button == 1) {
-		g_signal_emit (zoom_control, signals[ZOOM_TO_DEFAULT], 0);
 	}
-
 	/* We don't change our state (to reflect the new zoom) here.
 	   The zoomable will call back with the new level.
 	   Actually, the callback goes to the viewframe containing the
@@ -150,7 +225,7 @@ nautilus_zoom_control_button_press_event (GtkWidget *widget,
 	   calling into is - nautilus_zoom_control_set_zoom_level.
 	*/
 
-	return TRUE;
+	return FALSE;
 }
 
 static void
@@ -202,7 +277,7 @@ nautilus_zoom_control_instance_init (NautilusZoomControl *zoom_control)
 	GtkWidget *image;
 	int i;
 	
-	zoom_control->details = g_new0 (NautilusZoomControlDetails, 1);
+	zoom_control->details = G_TYPE_INSTANCE_GET_PRIVATE (zoom_control, NAUTILUS_TYPE_ZOOM_CONTROL, NautilusZoomControlDetails);
 
 	zoom_control->details->zoom_level = NAUTILUS_ZOOM_LEVEL_STANDARD;
 	zoom_control->details->min_zoom_level = NAUTILUS_ZOOM_LEVEL_SMALLEST;
@@ -210,14 +285,15 @@ nautilus_zoom_control_instance_init (NautilusZoomControl *zoom_control)
 	zoom_control->details->has_min_zoom_level = TRUE;
 	zoom_control->details->has_max_zoom_level = TRUE;
 
-	for (i = NAUTILUS_ZOOM_LEVEL_SMALLEST; i <= NAUTILUS_ZOOM_LEVEL_LARGEST; i++) {
-		zoom_control->details->preferred_zoom_levels = g_list_append (
+	for (i = NAUTILUS_ZOOM_LEVEL_LARGEST; i >= NAUTILUS_ZOOM_LEVEL_SMALLEST; i--) {
+		zoom_control->details->preferred_zoom_levels = g_list_prepend (
 		          zoom_control->details->preferred_zoom_levels,
 			  GINT_TO_POINTER (i));
 	}
 
 	image = gtk_image_new_from_stock (GTK_STOCK_ZOOM_OUT, GTK_ICON_SIZE_MENU);
 	zoom_control->details->zoom_out = gtk_button_new ();
+	gtk_button_set_focus_on_click (GTK_BUTTON (zoom_control->details->zoom_out), FALSE);
 	gtk_button_set_relief (GTK_BUTTON (zoom_control->details->zoom_out),
 			       GTK_RELIEF_NONE);
 	g_signal_connect (G_OBJECT (zoom_control->details->zoom_out),
@@ -227,18 +303,28 @@ nautilus_zoom_control_instance_init (NautilusZoomControl *zoom_control)
 	gtk_box_pack_start (GTK_BOX (zoom_control),
 			    zoom_control->details->zoom_out, FALSE, FALSE, 0);
 
-	zoom_control->details->zoom_event = gtk_event_box_new ();
-	g_signal_connect (G_OBJECT (zoom_control->details->zoom_event),
+	zoom_control->details->zoom_button = gtk_button_new ();
+	gtk_button_set_focus_on_click (GTK_BUTTON (zoom_control->details->zoom_button), FALSE);
+	gtk_button_set_relief (GTK_BUTTON (zoom_control->details->zoom_button),
+			       GTK_RELIEF_NONE);
+			       
+	gtk_widget_add_events (GTK_WIDGET (zoom_control->details->zoom_button),
+			       GDK_BUTTON_PRESS_MASK
+			       | GDK_BUTTON_RELEASE_MASK
+			       | GDK_POINTER_MOTION_MASK);		       
+			       
+	g_signal_connect (G_OBJECT (zoom_control->details->zoom_button),
 			  "button-press-event",
 			  G_CALLBACK (nautilus_zoom_control_button_press_event),
 			  zoom_control);
-	GTK_WIDGET_SET_FLAGS (zoom_control->details->zoom_event, GTK_CAN_FOCUS);
-	GTK_WIDGET_UNSET_FLAGS (zoom_control->details->zoom_event, GTK_NO_WINDOW);
-
-	gtk_widget_add_events (GTK_WIDGET (zoom_control->details->zoom_event),
-			       GDK_BUTTON_PRESS_MASK
-			       | GDK_BUTTON_RELEASE_MASK
-			       | GDK_POINTER_MOTION_MASK);
+			  		  		  
+	g_signal_connect (G_OBJECT (zoom_control->details->zoom_button),
+			  "clicked", G_CALLBACK (zoom_button_clicked),
+			  zoom_control);
+	
+	g_signal_connect (G_OBJECT (zoom_control->details->zoom_button),
+			  "popup-menu", G_CALLBACK (zoom_popup_menu),
+			  zoom_control);
 
 	zoom_control->details->zoom_label = gtk_label_new ("100%");
 	g_signal_connect (zoom_control->details->zoom_label,
@@ -247,13 +333,14 @@ nautilus_zoom_control_instance_init (NautilusZoomControl *zoom_control)
 			  zoom_control);
 	set_label_size (zoom_control);
 	
-	gtk_container_add (GTK_CONTAINER (zoom_control->details->zoom_event), zoom_control->details->zoom_label);
+	gtk_container_add (GTK_CONTAINER (zoom_control->details->zoom_button), zoom_control->details->zoom_label);
 
 	gtk_box_pack_start (GTK_BOX (zoom_control),
-			    zoom_control->details->zoom_event, TRUE, TRUE, 0);
+			    zoom_control->details->zoom_button, TRUE, TRUE, 0);
 
 	image = gtk_image_new_from_stock (GTK_STOCK_ZOOM_IN, GTK_ICON_SIZE_MENU);
 	zoom_control->details->zoom_in = gtk_button_new ();
+	gtk_button_set_focus_on_click (GTK_BUTTON (zoom_control->details->zoom_in), FALSE);
 	gtk_button_set_relief (GTK_BUTTON (zoom_control->details->zoom_in),
 			       GTK_RELIEF_NONE);
 	g_signal_connect (G_OBJECT (zoom_control->details->zoom_in),
@@ -265,7 +352,7 @@ nautilus_zoom_control_instance_init (NautilusZoomControl *zoom_control)
 			    zoom_control->details->zoom_in, FALSE, FALSE, 0);
 
 	gtk_widget_show_all (zoom_control->details->zoom_out);
-	gtk_widget_show_all (zoom_control->details->zoom_event);
+	gtk_widget_show_all (zoom_control->details->zoom_button);
 	gtk_widget_show_all (zoom_control->details->zoom_in);
 }
 
@@ -346,9 +433,6 @@ create_zoom_menu_item (NautilusZoomControl *zoom_control, GtkMenu *menu,
 	 */
 	zoom_control->details->marking_menu_items = TRUE;
 
-	/* This is marked for localization in case the % sign is not
-	 * appropriate in some locale. I guess that's unlikely.
-	 */
 	percent = floor ((100.0 * nautilus_get_relative_icon_size_for_zoom_level (zoom_level)) + .5);
 	item_text = g_strdup_printf ("%d%%", percent);
 
@@ -356,6 +440,7 @@ create_zoom_menu_item (NautilusZoomControl *zoom_control, GtkMenu *menu,
 		? NULL
 		: gtk_radio_menu_item_get_group (previous_radio_item);
 	menu_item = gtk_radio_menu_item_new_with_label (radio_item_group, item_text);
+	g_free (item_text);
 
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), 
 					zoom_level == zoom_control->details->zoom_level);
@@ -631,6 +716,8 @@ nautilus_zoom_control_class_init (NautilusZoomControlClass *class)
 				      "change_value",
 				      1, GTK_TYPE_SCROLL_TYPE,
 				      GTK_SCROLL_STEP_UP);
+
+	g_type_class_add_private (G_OBJECT_CLASS (class), sizeof (NautilusZoomControlDetails));
 }
 
 static gboolean
